@@ -1,6 +1,7 @@
 require 'uri'
 require 'librarian/puppet/util'
-require 'librarian/puppet/source/forge/repo'
+require 'librarian/puppet/source/forge/repo_v1'
+require 'librarian/puppet/source/forge/repo_v3'
 
 module Librarian
   module Puppet
@@ -11,6 +12,14 @@ module Librarian
         class << self
           LOCK_NAME = 'FORGE'
 
+          def default=(source)
+            @@default = source
+          end
+
+          def default
+            @@default
+          end
+
           def lock_name
             LOCK_NAME
           end
@@ -51,6 +60,12 @@ module Librarian
 
         def initialize(environment, uri, options = {})
           self.environment = environment
+
+          if uri =~ %r{^http(s)?://forge\.puppetlabs\.com}
+            uri = "https://forgeapi.puppetlabs.com"
+            warn { "Replacing Puppet Forge API URL to use v3 #{uri}. You should update your Puppetfile" }
+          end
+
           @uri = URI::parse(uri)
           @cache_path = nil
         end
@@ -114,7 +129,7 @@ module Librarian
         end
 
         def install_path(name)
-          environment.install_path.join(name.split('/').last)
+          environment.install_path.join(module_name(name))
         end
 
         def fetch_version(name, version_uri)
@@ -128,7 +143,7 @@ module Librarian
 
         def fetch_dependencies(name, version, version_uri)
           repo(name).dependencies(version).map do |k, v|
-            v = Requirement.new(v).gem_requirement
+            v = Librarian::Dependency::Requirement.new(v).to_gem_requirement
             Dependency.new(k, v, nil)
           end
         end
@@ -141,7 +156,16 @@ module Librarian
 
         def repo(name)
           @repo ||= {}
-          @repo[name] ||= Repo.new(self, name)
+
+          unless @repo[name]
+            # if we are using the official Forge then use API v3, otherwise stick to v1 for now
+            if uri.hostname =~ /\.puppetlabs\.com$/ || !environment.use_v1_api
+              @repo[name] = RepoV3.new(self, name)
+            else
+              @repo[name] = RepoV1.new(self, name)
+            end
+          end
+          @repo[name]
         end
       end
     end
